var   sys = require('sys'),
      querystring = require('querystring'),
      rest = require('../vendor/restler/lib/restler');
      
      
var base_path = '/api/v3',
    base_format = '.json';

function merge(defaults, options) {
	defaults = defaults || {};
	options = options || {};
	if (typeof options === 'object') {
		var keys = Object.keys(options);
		for (var i = 0, len = keys.length; i < len; i++) {
			var k = keys[i];
			if (options[k] !== undefined) defaults[k] = options[k];
		}
	}
	return defaults;
}

CreateSend = rest.service(function(api_key_or_username, password) {
    this.defaults.username = api_key_or_username;
    this.defaults.ignoreErrors = true;
    if (password) {
      this.defaults.password = password;
    } else {
      this.defaults.password = 'magic';
    }
    
  }, {
    baseURL: 'http://api.createsend.com'
  }, {
    /*
    
    */
    getClients: function(callback) {
      this.get(base_path + '/clients' + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    getCountries: function(callback) {
      this.get(base_path + '/countries' + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    getTimezones: function(callback) {
      this.get(base_path + '/timezones' + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    getSystemDate: function(callback) {
      this.get(base_path + '/systemdate' + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    
    
    // Client Functionality
    
    createClient: function(request_body, callback) {
      this.post(base_path + '/clients' + base_format, { jsondata: true, data: request_body } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        })
				.addListener('error', function(data, res) {
					console.log(data);
				});
    },
    
    getClientDetails: function(clientid, callback) {
      this.get(base_path + '/clients/' + clientid + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    getClientCampaigns: function(clientid, callback) {
      this.get(base_path + '/clients/' + clientid + '/campaigns' + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    getClientDrafts: function(clientid, callback) {
      this.get(base_path + '/clients/' + clientid + '/drafts' + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    getClientLists: function(clientid, callback) {
      this.get(base_path + '/clients/' + clientid + '/lists' + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    getClientSegments: function(clientid, callback) {
      this.get(base_path + '/clients/' + clientid + '/segments' + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    getClientSuppressionList: function(clientid, options, callback) {
      if (typeof options == 'function') {
        callback = options;
        options = {};
      }
      
      var url_options = {
        page: 1,
        pagesize: 1000,
        orderfield: 'email',
        orderdirection: 'asc'
      }
      
      url_options = merge(url_options, options);
      
      this.get(base_path + '/clients/' + clientid + '/suppressionlist' + base_format + '?' + querystring.stringify(url_options), { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    getClientTemplates: function(clientid, callback) {
      this.get(base_path + '/clients/' + clientid + '/templates' + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    setClientDetails: function(clientid, request_body, callback) {
      this.put(base_path + '/clients/' + clientid + '/setbasics' + base_format, { jsondata: true, data: request_body } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    setClientAccess: function(clientid, request_body, callback) {
      this.post(base_path + '/clients/' + clientid + '/setaccess' + base_format, { jsondata: true, data: request_body } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },
    
    deleteClient: function(clientid, callback) {
      this.del(base_path + '/clients/' + clientid + base_format, { } )
        .addListener('complete', function(data, res) {
          parseResponse(data, res, callback);
        });
    },

		// Subscriber Functionality
		
		addSubscriber: function(listid, options, callback) {
			var default_options = {
				'EmailAddress': '',
				'Name': '',
				'CustomFields': [],
				'Resubscribe': false
			};
			
			var request_body = merge(default_options, options);
			
			this.post(base_path + '/subscribers/' + listid + base_format, { jsondata: true, data: request_body } )
				.addListener('complete', function(data, res) {
					parseResponse(data, res, callback);
				});
		},
		importSubscribers: function(listid, list_array, callback) {
			// TODO: check if array
			var subscribers = [];			
			var request_body = {
				'Subscribers': list_array
			};
			
			this.post(base_path + '/subscribers/' + listid + '/import' + base_format, { jsondata: true, data: request_body } )
				.addListener('complete', function(data, res) {
					parseResponse(data, res, callback);
				});
		},
		getSubscriberDetails: function(listid, email_address, callback) {
			this.get(base_path + '/subscribers/' + listid + base_format, { query: { email: email_address} } )
				.addListener('complete', function(data, res) {
					parseResponse(data, res, callback);
				});
		},
		getSubscriberHistory: function(listid, email_address, callback) {
			this.get(base_path + '/subscribers/' + listid + '/history' + base_format, { query: { email: email_address} } )
				.addListener('complete', function(data, res) {
					parseResponse(data, res, callback);
				});
		},
		emailUnsubscribe: function(listid, email_address, callback) {
			var request_body = {
				'EmailAddress': email_address
			}
			this.post(base_path + '/subscribers/' + listid + '/unsubscribe' + base_format, { jsondata: true, data: request_body } )
				.addListener('complete', function(data, res) {
					parseResponse(data, res, callback);
				});
		}
    
  });

CreateSend.VERSION = '0.1.1';
module.exports = CreateSend;




function parseResponse(data, res, callback) {
  switch (res.statusCode) {
    
    case 200:
    case 201:
      // GET/POST/POST(Campaign Sent)/PUT/DELETE: Everything is good!
    
      if (callback) {
        if (!data) {
          callback(null, true);
        } else {
          callback(null, data);
        }
      }
      break;
      
    case 400:
    case 401:
    case 404:
    case 500:
      // Something went wrong, lets populate the error object
      
      if (callback) {
        callback(data, null);
      }
      break;
    
    default:
      throw Error('CM: Unknown status code returned.');
  
  }
    
}
