var fs = require('fs'),
    sys = require('sys'),
    path = require('path'),
    assert = require('assert'),
    events = require('events');

require.paths.unshift(path.join(__dirname, '..', 'lib'));

var vows = require('vows');

var CreateSend = require('createsend');

try {
  var test_data = JSON.parse(fs.readFileSync(__dirname + '/test_data', 'ascii'));
} catch (err) {
  sys.puts(sys.inspect(err));
  console.error('Testing requires ./test_data to contain a JSON string with');
  console.error('key in order to run tests.');
  process.exit(1);
}

vows.describe('CreateSend/Account').addBatch({
  'Campaign Monitor': {
    topic: new(CreateSend)(test_data.api_key),
    'connects using': {
      topic: function (cm) {
        cm.getSystemDate(this.callback);
      },
      'a valid api key': function(err,data) {
        assert.isNull (err);
        assert.isNotNull (data);
      }
    },
    'can get a list of countries': {
      topic: function (cm) {
        cm.getCountries(this.callback);
      },
      'has a valid response': function (err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
        assert.length (data, 245);
      },
      'includes valid data': function (err, data) {
        assert.include (data, 'United States of America');
        assert.include (data, 'Netherlands');
      }
    },
    'can get a list of time zones': {
      topic: function (cm) {
        cm.getTimezones(this.callback);
      },
      'has a valid response': function (err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
        assert.length (data, 98);
      },
      'includes valid data': function (err, data) {
        assert.include (data, '(GMT-08:00) Pacific Time (US & Canada)');
        assert.include (data, '(GMT+01:00) Amsterdam, Berlin, Bern, Rome, Stockholm, Vienna');
      }
    },
    'can get a list of clients': {
      topic: function (cm) {
        cm.getClients(this.callback);
      },
      'has a valid response': function (err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
        assert.isArray (data);
      },
      'includes the test client': function (err, data) {
        assert.include (data.ClientID, test_data.test_client); 
      }
    }
  }

}).export(module);
