var fs = require('fs'),
    sys = require('sys'),
    path = require('path'),
    assert = require('assert'),
    events = require('events');

require.paths.unshift(path.join(__dirname, '..', 'lib'));

var vows = require('vows');

var CreateSend = require('createsend');

try {
  var test_data = JSON.parse(fs.readFileSync(__dirname + '/test_data', 'ascii'));
} catch (err) {
  sys.puts(sys.inspect(err));
  console.error('Testing requires ./test_data to contain a JSON string with');
  console.error('key in order to run tests.');
  process.exit(1);
}

vows.describe('CreateSend/ClientCreate').addBatch({
  'A CreateSend instance': {
    topic: new(CreateSend)(test_data.api_key),
    'connects using': {
      topic: function (cm) {
        cm.getSystemDate(this.callback);
      },
      'a valid api key': function(err,data) {
        assert.isNull (err);
        assert.isNotNull (data);
      }
    },
    'can get a list of clients': {
      topic: function (cm) {
        cm.getClients(this.callback);
      },
      'has a valid response': function (err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
        assert.isArray (data);
      },
      'includes the test client': function (err, data) {
        assert.include (data.ClientID, test_data.test_client); 
      }
    },
    'can create a new client': {
      topic: function (cm) {
        var promise = new(events.EventEmitter);
        this.cm = cm;
        this.cm.createClient(test_data.new_client, function(err, data) {
          if (err) { promise.emit('error', err) }
          else { promise.emit('success', data) }
        });
        
        return promise;
      },
      'works as intented': function (err, data) {
        assert.isNull(err);
        assert.isNotNull(data);
        this.new_client_id = data;
      },
      'that can be retreived': {
        topic: function(err, data) {
          this.cm.getClientDetails(this.new_client_id, this.callback);
        },
        'with the correct values': function(err, data) {
          assert.equal (data.BasicDetails.EmailAddress, test_data.new_client.EmailAddress);
        },
        'to modify basic details': {
          topic: function() {
            this.cm.setClientDetails(this.new_client_id, test_data.new_client_revised, this.callback);
          },
          'works as intented': function (err, data) {
            assert.isNull (err);
          },
					'then deleted': {
						topic: function() {
							this.cm.deleteClient(this.new_client_id, this.callback)
						},
						'works as intended': function (err, data) {
							assert.isNull (err);
						}
					}
        }
      }
    }
  }

}).export(module);
