var fs = require('fs'),
    sys = require('sys'),
    path = require('path'),
    assert = require('assert'),
    events = require('events');

require.paths.unshift(path.join(__dirname, '..', 'lib'));

var vows = require('vows');

var CreateSend = require('createsend');

try {
  var test_data = JSON.parse(fs.readFileSync(__dirname + '/test_data', 'ascii'));
} catch (err) {
  sys.puts(sys.inspect(err));
  console.error('Testing requires ./test_data to contain a JSON string with');
  console.error('key in order to run tests.');
  process.exit(1);
}

vows.describe('CreateSend/ClientsSummarize').addBatch({
  'Campaign Monitor:': {
    topic: new(CreateSend)(test_data.api_key),
    'connects using': {
      topic: function (cm) {
        cm.getSystemDate(this.callback);
      },
      'a valid api key': function(err,data) {
        assert.isNull (err);
        assert.isNotNull (data);
      }
    },
    'when getting a list of clients': {
      topic: function (cm) {
        cm.getClients(this.callback);
      },
      'response is valid': function (err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
        assert.isArray (data);
      },
      'includes the test client': function (err, data) {
        assert.include (data.ClientID, test_data.test_client); 
      }
    },
    'when getting test client\'s details': {
      topic: function (cm) {
        var promise = new(events.EventEmitter);
        
        cm.getClientDetails(test_data.test_client, function(err, data) {
          if (err) { promise.emit('error', err) }
          else { promise.emit('success', data) };
        });        
        
        return promise;
      },
      'details exist': function(err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
      }
    },
    'when getting a list of sent campaigns': {
      topic: function (cm) {
        var promise = new(events.EventEmitter);
        
        cm.getClientCampaigns(test_data.test_client, function(err, data) {
          if (err) { promise.emit('error', err) }
          else { promise.emit('success', data) };
        });        
        
        return promise;
      },
      'details exist': function(err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
      }
    },
    'when getting a list of draft campaigns': {
      topic: function (cm) {
        var promise = new(events.EventEmitter);
        
        cm.getClientDrafts(test_data.test_client, function(err, data) {
          if (err) { promise.emit('error', err) }
          else { promise.emit('success', data) };
        });        
        
        return promise;
      },
      'details exist': function(err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
      }
    },
    'when getting a list of draft campaigns': {
      topic: function (cm) {
        var promise = new(events.EventEmitter);
        
        cm.getClientDrafts(test_data.test_client, function(err, data) {
          if (err) { promise.emit('error', err) }
          else { promise.emit('success', data) };
        });        
        
        return promise;
      },
      'details exist': function(err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
      }
    }
  }

}).export(module);
