var fs = require('fs'),
    sys = require('sys'),
    path = require('path'),
    assert = require('assert'),
    events = require('events');

require.paths.unshift(path.join(__dirname, '..', 'lib'));

var vows = require('vows');

var CreateSend = require('createsend');

try {
  var test_data = JSON.parse(fs.readFileSync(__dirname + '/test_data', 'ascii'));
} catch (err) {
  sys.puts(sys.inspect(err));
  console.error('Testing requires ./test_data to contain a JSON string with');
  console.error('key in order to run tests.');
  process.exit(1);
}

vows.describe('CreateSend/Subscribers').addBatch({
  'A CreateSend instance': {
    topic: new(CreateSend)(test_data.api_key),
    'connects using': {
      topic: function (cm) {
        cm.getSystemDate(this.callback);
      },
      'a valid api key': function(err,data) {
        assert.isNull (err);
        assert.isNotNull (data);
      }
    },
    'can add a subscriber': {
      topic: function (cm) {
        cm.addSubscriber(test_data.test_list, test_data.new_subscriber, this.callback);
				this.cm = cm;
      },
      'has a valid response': function (err, data) {
        assert.isNull (err);
        assert.isNotNull (data);
      },
      'includes valid data': function (err, data) {
        assert.equal (data, test_data.new_subscriber['EmailAddress']);
      },
			'and can get subscriber details': {
				topic: function () {
					this.cm.getSubscriberDetails(test_data.test_list, test_data.new_subscriber['EmailAddress'], this.callback);
				},
				'has a valid response': function(err, data) {
					assert.isNull (err);
					assert.isNotNull (data);
				},
				'includes correct data': function (err, data) {
					assert.equal (data['Name'], test_data.new_subscriber['Name']);
				}
			},
			'and can get subscriber history': {
				topic: function () {
					this.cm.getSubscriberHistory(test_data.test_list, test_data.new_subscriber['EmailAddress'], this.callback);
				},
				'has a valid response': function (err, data) {
					assert.isNull (err);
					assert.isNotNull (data);
				}
			},
			'and can unsubscribe': {
				topic: function() {
					this.cm.emailUnsubscribe(test_data.test_list, test_data.new_subscriber['EmailAddress'], this.callback);
				},
				'has a valid response': function (err, data) {
					assert.isNull (err);
					assert.isNotNull (data);
				}
			}
    },
		'can add multiple subscribers': {
			topic: function (cm) {
				cm.importSubscribers(test_data.test_list, test_data.import_subscribers, this.callback);
			},
			'has a valid response': function (err, data) {
				assert.isNull (err);
				assert.isNotNull (data);
			}
		}
  }

}).export(module);
